#ifndef OPENGL_SHADER_H
#define OPENGL_SHADER_H

#include <string>
#include <epoxy/gl.h>
#include <glm/glm.hpp>

class OpenGlShaderProgram
{
	public:
		OpenGlShaderProgram(std::string vertex_file, std::string geometry_file, std::string fragment_file);
		~OpenGlShaderProgram();
		void setProjectionView(const glm::mat4 &projection, const glm::mat4 &view);
		glm::mat4 getProjection();
		glm::mat4 getView();
		int compile();
		void unload();
		int use();
		int getUniformLoc(const std::string &uniform);
		void setUniformFloat(const std::string &uniform, float value);
		void setUniformVec2(const std::string &uniform, float *vector);
		void setUniformMat4(const std::string &uniform, const glm::mat4 &matrix);
	private:
		glm::mat4 proj;
		glm::mat4 view;
		glm::mat4 proj_view;
		bool compiled;
		GLuint shader_program;
		std::string vertex_file;
		std::string geometry_file;
		std::string fragment_file;
};

#endif // OPENGL_SHADER_H
